//  Nylox
//  Copyright (C) 2006-2008 Agustin Alvarez
// 
//  This program is free software; you can redistribute it and/or modify it
//  under the terms of the GNU General Public License as published by the Free
//  Software Foundation; either version 2 of the License, or (at your option)
//  any later version.
// 
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
//  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
//  for more details.
//  
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//  CPU Related Stuff

#include <Libc.h>
#include <Arch\IA32\Hal.h>
#include "Includes/Cpu.h"
#include "Includes/Pit.h"
#include "Includes/Pic.h"
#include "Includes/Descriptor.h"

static unsigned long Delay_Count = 1; 
static volatile unsigned long CalTicks  = 0;

void i386PITIsrCal();

void i386PITCal()
{
 //! Incremental Tick Count
 CalTicks++;
 //! Tell HAL We Are Done
 HALIrqDone(0);     
}

asm (
   ".globl _i386PITIsrCal  \n"
   "_i386PITIsrCal:        \n"
   "   pusha               \n"
   "   pushw %ds           \n"
   "   pushw %es           \n"
   "   pushw %ss           \n" 
   "   pushw %ss           \n"
   "   popw %ds            \n"
   "   popw %es            \n"
   "                       \n"
   "   call _i386PITCal    \n"
   "                       \n"
   "   popw %es            \n"
   "   popw %ds            \n"
   "   popa                \n"
   "   iret                \n" 
);

int i386CPUInitialize()
{ 
 if(i386IDTInitialize() == 0x0) return 0x0;
 if(i386GDTInitialize() == 0x0) return 0x0;
 
 return 0x1;
}

int i386CPUShutdown()
{
 return 0x0;
}



/**  CPU Calibrate Clock **/
void i386CPUDelay(unsigned long Loops)
{
 unsigned long C;
 for(C=0;C<Loops;C++);
}

unsigned long i386CPUCalibratePFC()
{
 unsigned long prevtick;          /* Temporary variable             */
 unsigned int i;                  /* Counter variable               */
 unsigned int calib_bit;          /* Bit to calibrate (see below)   */
 
 
 HALInterruptSet(32,I86_IDT_DESC_PRESENT | I86_IDT_DESC_BIT32,0x8,&i386PITIsrCal);
 i386PICEnableIRQ(0);
 
 i386PITStartCounter(100,0,6);
 
 HalEnableInterrumpt();
 
 /* Stage 1:  Coarse calibration */
 do {
     Delay_Count <<= 1;                         /* Next delay count to try        */ 
     prevtick = CalTicks;               /* Wait for the start of the next */ 
     
     while(prevtick == CalTicks);       /* timer tick                     */ 
     
     prevtick = CalTicks;               /* Start measurement now          */ 
      i386CPUDelay(Delay_Count);                /* Do the delay                   */
    }while(prevtick == CalTicks);       /* Until delay is just too big    */ 
    
    Delay_Count <<= 1;                          /* Get bottom value for delay     */ 
 
 /* Stage 2:  Fine calibration   */
 calib_bit = Delay_Count;                       /* Which bit are we going to test */ 
 
 for(i=0;i<8;i++) {
     calib_bit >>= 1;                           /* Next bit to calibrate          */ 
     if(!calib_bit) break;                      /* If we have done all bits, stop */ 
     
     Delay_Count |= calib_bit;                  /* Set the bit in delay_count     */
     
     prevtick = CalTicks;               /* Wait for the start of the next */ 
     while(prevtick == CalTicks);       /* timer tick                     */ 
     
     prevtick = CalTicks;               /* Start measurement now          */ 
     i386CPUDelay(Delay_Count);                 /* Do the delay                   */
     
     if(prevtick != CalTicks)           /* If a tick has passed, turn the */
       Delay_Count &= ~calib_bit;               /* calibrated bit back off */
 }
 
 Delay_Count /= 10;                             /* Calculate delay_count for 1ms   */
 
 HalDisableInterrumpt();                        /* Back To Known State.. */
 
 return Delay_Count;
}

unsigned long i386CPUGetDelayCount()
{
 return Delay_Count;
}
