//  Nylox
//  Copyright (C) 2006-2008 Agustin Alvarez
// 
//  This program is free software; you can redistribute it and/or modify it
//  under the terms of the GNU General Public License as published by the Free
//  Software Foundation; either version 2 of the License, or (at your option)
//  any later version.
// 
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
//  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
//  for more details.
//  
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//  Hardware Abstraction Layer

#include <Libc.h>

#include "Includes/cpu.h"
#include "Includes/pic.h"
#include "Includes/pit.h"
#include "Includes/Descriptor.h"


int HALInitialize()
{
 
 if(i386CPUInitialize() == 0x0) return 0x0; 	/** Initialize CPU (Gdt,Idt)**/

 i386PICInitialize(0x20,0x28);					/** Initialize Programmable Interrumpt Control **/ 
 //i386CPUCalibratePFC();
 
 i386PITInitialize();							/** Initialize Programmable Interrumpt Timer **/
 i386PITStartCounter(100,0,6);					/** Counter 0,SquareWaveGen Mode **/


 return 0x1;
}

void HalEnableInterrumpt()
{
 kernelProcessorEnableInts();
}

void HalDisableInterrumpt()
{
 kernelProcessorDisableInts();
}

int HALShutdown()
{
 return 0x0;
}

void HALIrqDone(unsigned int IRQ)
{
 //! insure its a valid hardware irq
 if(IRQ > 16) return;
 //! test if we need to send end-of-interrupt to second pic
 if(IRQ > 8) kernelProcessorOutPort8(0xA0,0x20);
 //! always send end-of-interrupt to primary pic
 kernelProcessorOutPort8(0x20,0x20);
}

 void  HALIrqEnable(unsigned int IRQ)
 {
  i386PICEnableIRQ(IRQ);
 }      
 void  HALIrqDisable(unsigned int IRQ)
 {
  i386PICDisableIRQ(IRQ);
 }

int HALInterruptSet(unsigned int INT,unsigned short Flags,unsigned short Sel,void (*Func))
{
 return i386IDTSet(INT,Flags,Sel,Func);
}

void HalPerfomanceDelay(unsigned long Milliseconds)
{
 i386CPUDelay(Milliseconds*i386CPUGetDelayCount());       /* Delay milliseconds ms */ 
}
 
