//
//  Nylox
//  Copyright (C) 2006-2008 Agustin Alvarez
// 
//  This program is free software; you can redistribute it and/or modify it
//  under the terms of the GNU General Public License as published by the Free
//  Software Foundation; either version 2 of the License, or (at your option)
//  any later version.
// 
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
//  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
//  for more details.
//  
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//  K32Descriptor.h
//

#if !defined(_Descriptor_H)

// Definitions
#define PRIV_CODE       0x00000008
#define PRIV_DATA       0x00000010
#define PRIV_STACK      0x00000018

//! must be in the format 0D110, where D is descriptor type
#define I86_IDT_DESC_BIT16			0x06	//00000110
#define I86_IDT_DESC_BIT32			0x0E	//00001110
#define I86_IDT_DESC_RING1			0x40	//01000000
#define I86_IDT_DESC_RING2			0x20	//00100000
#define I86_IDT_DESC_RING3			0x60	//01100000
#define I86_IDT_DESC_PRESENT		0x80	//10000000

//! set access bit
#define I86_GDT_DESC_ACCESS				0x0001			//00000001
//! descriptor is readable and writable. default: read only
#define I86_GDT_DESC_READWRITE			0x0002			//00000010
//! set expansion direction bit
#define I86_GDT_DESC_EXPANSION			0x0004			//00000100
//! executable code segment. Default: data segment
#define I86_GDT_DESC_EXEC_CODE			0x0008			//00001000
//! set code or data descriptor. defult: system defined descriptor
#define I86_GDT_DESC_CODEDATA			0x0010			//00010000
//! set dpl bits
#define I86_GDT_DESC_DPL				0x0060			//01100000
//! set "in memory" bit
#define I86_GDT_DESC_MEMORY				0x0080			//10000000
/**		gdt descriptor grandularity bit flags	***/
//! masks out limitHi (High 4 bits of limit)
#define I86_GDT_GRAND_LIMITHI_MASK		0x0f			//00001111
//! set os defined bit
#define I86_GDT_GRAND_OS				0x10			//00010000
//! set if 32bit. default: 16 bit
#define I86_GDT_GRAND_32BIT				0x40			//01000000
//! 4k grandularity. default: none
#define I86_GDT_GRAND_4K				0x80			//10000000

#define RES_GLOBAL_DESCRIPTORS 8  // (0 is unusable)
#define GDT_SIZE 1024
#define IDT_SIZE 256

// Results
#define GDT_ALLOC_SUCCESFULL 1
#define IDT_ALLOC_SUCCESFULL 1

#define IDT_ALLOC_INVALID	 0
#define GDT_ALLOC_INVALID 	 0

// Traps, IDT Definitions
#define TRAP_STOP_EXECUTION 1
#define TRAP_CONTINUE_EXECUTION 0

typedef int i386_Selector;

// This structure describes the fields of a descriptor in the x86 architecture
typedef struct {

	unsigned short	BaseL;
	unsigned short	Sel;
	unsigned char 	Reserved;
	unsigned char 	Flags;
	unsigned short	BaseH;
} i386_IDTDescriptor;

typedef struct{

	//! bits 0-15 of segment limit
	unsigned short	Limit;
	//! bits 0-23 of base address
	unsigned short	BaseL;
	unsigned char	BaseM;
	//! descriptor access flags
	unsigned char	Flags;
	unsigned char	Grand;
	//! bits 24-32 of base address
	unsigned char	BaseH;
}i386_GDTDescriptor;


// Functions
int i386GDTInitialize(); int i386IDTInitialize(); //Constructor,Killer VFuncs

int i386GDTSet(volatile i386_Selector Selector, volatile void *Base,unsigned Size,unsigned short Access,unsigned short Gran);
int i386IDTSet(volatile i386_Selector INTNumber,unsigned short Flags,unsigned short Sel,void (*Address));


#define _Descriptor_H
#endif
