//
//  Nylox
//  Copyright (C) 2006-2008 Agustin Alvarez
// 
//  This program is free software; you can redistribute it and/or modify it
//  under the terms of the GNU General Public License as published by the Free
//  Software Foundation; either version 2 of the License, or (at your option)
//  any later version.
// 
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
//  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
//  for more details.
//  
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
// K32Pic.c
//
// Programmable Interval Controller

#include "Includes/Pic.h"
#include <Libc.h>

//! PIC 1 register port addresses
#define I86_PIC1_REG_COMMAND	0x20
#define I86_PIC1_REG_STATUS		0x20
#define I86_PIC1_REG_DATA		0x21
#define I86_PIC1_REG_IMR		0x21

//! PIC 2 register port addresses
#define I86_PIC2_REG_COMMAND	0xA0
#define I86_PIC2_REG_STATUS		0xA0
#define I86_PIC2_REG_DATA		0xA1
#define I86_PIC2_REG_IMR		0xA1

//-----------------------------------------------
//	Initialization Command Bit Masks
//-----------------------------------------------

//! Initialization Control Word 1 bit masks
#define I86_PIC_ICW1_MASK_IC4			0x1			//00000001
#define I86_PIC_ICW1_MASK_SNGL			0x2			//00000010
#define I86_PIC_ICW1_MASK_ADI			0x4			//00000100
#define I86_PIC_ICW1_MASK_LTIM			0x8			//00001000
#define I86_PIC_ICW1_MASK_INIT			0x10		//00010000

//! Initialization Control Words 2 and 3 do not require bit masks

//! Initialization Control Word 4 bit masks
#define I86_PIC_ICW4_MASK_UPM			0x1			//00000001
#define I86_PIC_ICW4_MASK_AEOI			0x2			//00000010
#define I86_PIC_ICW4_MASK_MS			0x4			//00000100
#define I86_PIC_ICW4_MASK_BUF			0x8			//00001000
#define I86_PIC_ICW4_MASK_SFNM			0x10		//00010000

//-----------------------------------------------
//	Initialization Command 1 control bits
//-----------------------------------------------

#define I86_PIC_ICW1_IC4_EXPECT				1			//1
#define I86_PIC_ICW1_IC4_NO					0			//0
#define I86_PIC_ICW1_SNGL_YES				2			//10
#define I86_PIC_ICW1_SNGL_NO				0			//00
#define I86_PIC_ICW1_ADI_CALLINTERVAL4		4			//100
#define I86_PIC_ICW1_ADI_CALLINTERVAL8		0			//000
#define I86_PIC_ICW1_LTIM_LEVELTRIGGERED	8			//1000
#define I86_PIC_ICW1_LTIM_EDGETRIGGERED		0			//0000
#define I86_PIC_ICW1_INIT_YES				0x10		//10000
#define I86_PIC_ICW1_INIT_NO				0			//00000

//-----------------------------------------------
//	Initialization Command 4 control bits
//-----------------------------------------------

#define I86_PIC_ICW4_UPM_86MODE			1			//1
#define I86_PIC_ICW4_UPM_MCSMODE		0			//0
#define I86_PIC_ICW4_AEOI_AUTOEOI		2			//10
#define I86_PIC_ICW4_AEOI_NOAUTOEOI		0			//0
#define I86_PIC_ICW4_MS_BUFFERMASTER	4			//100
#define I86_PIC_ICW4_MS_BUFFERSLAVE		0			//0
#define I86_PIC_ICW4_BUF_MODEYES		8			//1000
#define I86_PIC_ICW4_BUF_MODENO			0			//0
#define I86_PIC_ICW4_SFNM_NESTEDMODE	0x10		//10000
#define I86_PIC_ICW4_SFNM_NOTNESTED		0			//a binary 2 (futurama joke hehe ;)

void i386_PicSendCmd(unsigned char Cmd,unsigned char PicN)
{
 if(PicN > 0x1) return;
 
 unsigned char Reg = (PicN == 0x1) ? I86_PIC2_REG_COMMAND : I86_PIC1_REG_COMMAND;
 kernelProcessorOutPort8(Reg,Cmd);
}

void i386_PicSendData(unsigned char Data,unsigned char PicN)
{
 if(PicN > 0x1) return;
 
 unsigned char Reg = (PicN == 0x1) ? I86_PIC2_REG_DATA : I86_PIC1_REG_DATA;
 kernelProcessorOutPort8(Reg,Data);
}

unsigned char i386_PicReadData(unsigned char PicN)
{
 
 if(PicN > 0x1) return 0x0;
 
 unsigned char Reg = (PicN == 0x1) ? I86_PIC2_REG_DATA : I86_PIC1_REG_DATA;
 unsigned char Rec = 0;
 kernelProcessorOutPort8(Reg,Rec);
 
 return Rec;
}

void i386PICInitialize(unsigned char BASE0,unsigned char BASE1)
{
    /*
		unsigned char ICW = 0;

		ICW = (ICW & ~I86_PIC_ICW1_MASK_INIT) | I86_PIC_ICW1_INIT_YES;
		ICW = (ICW & ~I86_PIC_ICW1_MASK_IC4) | I86_PIC_ICW1_IC4_EXPECT;
		
		i386_PicSendCmd(ICW,0x0);
		i386_PicSendCmd(ICW,0x1);
		i386_PicSendData(BASE0,0x0);
		i386_PicSendData(BASE1,0x1);
		i386_PicSendData(0x04,0x0);
		i386_PicSendData(0x02,0x1);
		//! Send Initialization control word 4. Enables i86 mode
		ICW = (ICW & ~I86_PIC_ICW4_MASK_UPM) | I86_PIC_ICW4_UPM_86MODE;
		
		i386_PicSendData(ICW,0x0);
		i386_PicSendData(ICW,0x1);
	*/
	kernelProcessorOutPort8(0x20, 0x11);
	kernelProcessorOutPort8(0xA0, 0x11);

	kernelProcessorOutPort8(0x21, 0x20);
	kernelProcessorOutPort8(0xA1, 0x28);

	kernelProcessorOutPort8(0x21, 4);
	kernelProcessorOutPort8(0xA1, 2);

	kernelProcessorOutPort8(0x21, 0x01);
	kernelProcessorOutPort8(0xA1, 0x01);
	
	kernelProcessorOutPort8(0x21, 0x00);
	kernelProcessorOutPort8(0xA1, 0x00);
}


void i386PICEnableIRQ(unsigned char Number)
{
	BYTE MASK = 0;
	
	if(Number > 15) return;
	
	if(Number < 8) {
	   kernelProcessorInPort8(0x21,MASK);
	   MASK &= !(1 << Number);
	   kernelProcessorOutPort8(0x21,MASK);
	}else {
	   kernelProcessorInPort8(0xA1,MASK);
	   MASK &= !(1 << (Number-8));
	   kernelProcessorOutPort8(0xA1,MASK);
	}
}

void i386PICDisableIRQ(unsigned char Number)
{
	BYTE MASK = 0;
	
	if(Number > 15) return;
	
	if(Number < 8) {
	   kernelProcessorInPort8(0x21,MASK);
	   MASK |= 1 << Number;
	   kernelProcessorOutPort8(0x21,MASK);
	}else {
	   kernelProcessorInPort8(0xA1,MASK);
	   MASK |= 1 << (Number-8);
	   kernelProcessorOutPort8(0xA1,MASK);
	}
}
