//
//  Nylox
//  Copyright (C) 2006-2008 Agustin Alvarez
// 
//  This program is free software; you can redistribute it and/or modify it
//  under the terms of the GNU General Public License as published by the Free
//  Software Foundation; either version 2 of the License, or (at your option)
//  any later version.
// 
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
//  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
//  for more details.
//  
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
// K32Pic.c
//
// Programmable Interval Timer

#include "Includes/Pit.h"
#include "Includes/Pic.h"
#include "Includes/Descriptor.h"
#include <Arch\IA32\Hal.h>
#include <Libc.h>

//-----------------------------------------------
//	Controller Registers
//-----------------------------------------------

#define		I86_PIT_REG_COUNTER0		0x40
#define		I86_PIT_REG_COUNTER1		0x41
#define		I86_PIT_REG_COUNTER2		0x42
#define		I86_PIT_REG_COMMAND			0x43

static volatile unsigned char m_PitIsInit = 0;
static volatile unsigned long m_PitTicks  = 0;

void i386PITIsr();
void i386PITIrq();

asm (
   ".globl _i386PITIsr     \n"
   "_i386PITIsr:           \n"
   "   pusha               \n"
   "   pushw %ds           \n"
   "   pushw %es           \n"
   "   pushw %ss           \n" 
   "   pushw %ss           \n"
   "   popw %ds            \n"
   "   popw %es            \n"
   "                       \n"
   "   call _i386PITIrq    \n"
   "                       \n"
   "   popw %es            \n"
   "   popw %ds            \n"
   "   popa                \n"
   "   iret                \n" 
);

void i386PITIrq()
{ 
 //! Incremental Tick Count
 m_PitTicks++;
 //! Tell HAL We Are Done
 HALIrqDone(0);
}

//! Sets new pit tick count and returns prev. value
unsigned long i386PITSetTick(unsigned long Tick)
{
 unsigned long rET = m_PitTicks;
 m_PitTicks = Tick;
 return rET;
}
//! returns current tick count
unsigned long i386PITGetTick()
{
 return m_PitTicks;
}
//! send command to Pit
void i386PITSendCmd(unsigned char Cmd)
{
 kernelProcessorOutPort8(I86_PIT_REG_COMMAND,Cmd);
}
//! send data to a counter
void i386PITSendData(unsigned short Data,unsigned char Counter)
{
 unsigned char Port = (Counter==I86_PIT_OCW_COUNTER_0) ? I86_PIT_REG_COUNTER0 :
					  ((Counter==I86_PIT_OCW_COUNTER_1) ? I86_PIT_REG_COUNTER1 : I86_PIT_REG_COUNTER2);
 kernelProcessorOutPort8(Port,(unsigned char)Data);
}
//! read data from counter
unsigned char i386PITReadData(unsigned short Counter)
{
 unsigned char Port = (Counter==I86_PIT_OCW_COUNTER_0) ? I86_PIT_REG_COUNTER0 :
					  ((Counter==I86_PIT_OCW_COUNTER_1) ? I86_PIT_REG_COUNTER1 : I86_PIT_REG_COUNTER2);
 unsigned char rET;
 kernelProcessorInPort8(Port,rET);
 return rET;
}

void i386PITStartCounter(unsigned long Freq,unsigned char Counter,unsigned char Mode)
{
 if(Freq == 0x0) return;
 
 unsigned short Divisor = (unsigned short)(1193181 / (unsigned short)Freq);
 
 //! send operational command
 unsigned char OCW = 0x0;
 OCW = (OCW & ~I86_PIT_OCW_MASK_MODE) | Mode;
 OCW = (OCW & ~I86_PIT_OCW_MASK_RL) | I86_PIT_OCW_RL_DATA;
 OCW = (OCW & ~I86_PIT_OCW_MASK_COUNTER) | Counter;
 i386PITSendCmd (OCW);
 //! set frequency rate
 i386PITSendData (Divisor & 0xff, 0);
 i386PITSendData ((Divisor >> 8) & 0xff, 0); 
 //! reset tick count
 m_PitTicks = 0x0;
}

void i386PITInitialize()
{
 HALInterruptSet(32,I86_IDT_DESC_PRESENT | I86_IDT_DESC_BIT32,0x8,&i386PITIsr);
 i386PICEnableIRQ(0);
 
 m_PitIsInit = 0x1;
}

unsigned char i386PITStatus()
{
 return m_PitIsInit;
}
