//
//  Nylox
//  Copyright (C) 2006-2008 Agustin Alvarez
// 
//  This program is free software; you can redistribute it and/or modify it
//  under the terms of the GNU General Public License as published by the Free
//  Software Foundation; either version 2 of the License, or (at your option)
//  any later version.
// 
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
//  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
//  for more details.
//  
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//  K32Descriptor.h
//

#if !defined(_MEMORY_H)

#include <Libc.h>
#include <Exception.h>

#define PHYSICAL_RESERVED 0x20		//00100000
#define PHYSICAL_USED	  0x40		//01000000
#define PHYSICAL_FREE	  0x60		//01100000
#define PHYSICAL_PRESENT  0x80		//10000000

typedef struct
{
 unsigned long ExtendedMemory;
 
 void*		   MemoryHeap;
 unsigned long MemoryHeapSize;

 unsigned long MemoryTotal;
 unsigned long MemoryFree;
}Physical_Map_T;

typedef struct
{
 unsigned long Addr;
 unsigned long Size;
 unsigned char Property;
 
 void		  *Next;
}Physical_Index_T;

typedef struct Page
{
   unsigned long Present    : 1;   // Page present in memory
   unsigned long RW         : 1;   // Read-only if clear, readwrite if set
   unsigned long User       : 1;   // Supervisor level only if clear
   unsigned long Accessed   : 1;   // Has the page been accessed since last refresh?
   unsigned long Dirty      : 1;   // Has the page been written to since last refresh?
   unsigned long Unused     : 7;   // Amalgamation of unused and reserved bits
   unsigned long Frame      : 20;  // Frame address (shifted right 12 bits)
} Page_T;

typedef struct Page_Table
{
   Page_T Pages[1024];
} Page_Table_T;

typedef struct Page_Directory
{
   Page_Table_T *Tables[1024];

   unsigned long TablesPhy[1024];
   unsigned long PhysicalAddr;
} Page_Directory_T;

void mm_Arch_PhysicalInit(DWORD ExtendedMemory,DWORD KernelStart,DWORD KernelEnd);
void mm_Arch_PhysicalFreePage(DWORD PageN);
void mm_Arch_PhysicalAllocPage(DWORD PageN);
BYTE mm_Arch_PhysicalIsPageFree(DWORD PageN);
DWORD mm_Arch_PhysicalFindFreePage(DWORD Start);
DWORD mm_Arch_PhysicalFindFreeRangePage(DWORD Start,DWORD Size);

void mm_Arch_VirtualFreeFrame(Page_T* Page);
void mm_Arch_VirtualAllocFrame(Page_T* Page,int PVL,int RW);
Page_T* mm_Arch_VirtualGetPage(DWORD Address,int Make,Page_Directory_T *Dir);
void mm_Arch_VirtualInit(DWORD ExtendedMemory);
void mm_Arch_SwitchPageDirectory(Page_Directory_T* Dir);
void mm_Arch_VirtualPageFault(TRAP_FRAME TF);

void* mm_Arch_Malloc(DWORD Size,BYTE Align,DWORD* Phys);
void  mm_Arch_Free(void* P);

#define _MEMORY_H
#endif
