//
//  Nylox
//  Copyright (C) 2006-2008 Agustin Alvarez
// 
//  This program is free software; you can redistribute it and/or modify it
//  under the terms of the GNU General Public License as published by the Free
//  Software Foundation; either version 2 of the License, or (at your option)
//  any later version.
// 
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
//  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
//  for more details.
//  
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//  K32Descriptor.h
//

#if !defined(_K32Boot_H)

/* The magic number for the Multiboot header.  */
#define MULTIBOOT_HEADER_MAGIC		0x1BADB002

/* The flags for the Multiboot header.  */
#ifdef __ELF__
# define MULTIBOOT_HEADER_FLAGS		0x00000003
#else
# define MULTIBOOT_HEADER_FLAGS		0x00010003
#endif

/* The Multiboot header.  */
typedef struct Multiboot_Header
{
  unsigned long Magic;
  unsigned long Flags;
  unsigned long Checksum;
  unsigned long Header_addr;
  unsigned long Load_addr;
  unsigned long Load_end_addr;
  unsigned long Bss_end_addr;
  unsigned long Entry_addr;
} Multiboot_Header_T;

/* The symbol table for a.out.  */
typedef struct Aout_Symbol_Table
{
  unsigned long Tabsize;
  unsigned long Strsize;
  unsigned long Addr;
  unsigned long Reserved;
} Aout_Symbol_Table_T;

/* The section header table for ELF.  */
typedef struct Elf_Section_Header_Table
{
  unsigned long Num;
  unsigned long Size;
  unsigned long Addr;
  unsigned long Shndx;
} Elf_Section_Header_Table_T;

/* The Multiboot information.  */
typedef struct Multiboot_Info
{
  unsigned long Flags;
  unsigned long Mem_Lower;
  unsigned long Mem_Upper;
  unsigned long Boot_Device;
  unsigned long Cmdline;
  unsigned long Mods_Count;
  unsigned long Mods_Addr;
  union
  {
    Aout_Symbol_Table_T Aout_Sym;
    Elf_Section_Header_Table_T Elf_Sec;
  } u;
  unsigned long Mmap_Length;
  unsigned long Mmap_Addr;
} Multiboot_Info_T;

/* The module structure.  */
typedef struct Multiboot_Module
{
  unsigned long Mod_Start;
  unsigned long Mod_End;
  unsigned long String;
  unsigned long Reserved;
} Multiboot_Module_T;

/* The memory map. Be careful that the offset 0 is base_addr_low
   but no size.  */
typedef struct memory_map
{
  unsigned int Size;
  unsigned int Base_addr_low;
  unsigned int Base_addr_high;
  unsigned int Length_low;
  unsigned int Length_high;
  unsigned int Type;
} Memory_Map_T;


#define _K32Boot_H
#endif
