//
//  Nylox
//  Copyright (C) 2006-2008 Agustin Alvarez
// 
//  This program is free software; you can redistribute it and/or modify it
//  under the terms of the GNU General Public License as published by the Free
//  Software Foundation; either version 2 of the License, or (at your option)
//  any later version.
// 
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
//  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
//  for more details.
//  
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//  K32Timer.h
//

#if !defined(_libcmemory_H)
  
   #include "OrderedArray.h"
   
 /**Memory References **/
   //KMalloc Functions (Placement)
   DWORD KMalloc(DWORD Size);
   DWORD KMalloc_P(DWORD Size,DWORD *Phys);
   DWORD KMalloc_AP(DWORD Size,DWORD *Phys);
   DWORD KMalloc_A(DWORD Size);
   
   //Basic Functions
   void* Memcpy(void *Dest,const void *Src,unsigned Bytes);
   void* Memsetb(void* Dst,int Val,unsigned Bytes);
   void* Memsetw(void* Dst,int Val,unsigned Bytes);
   void* Memsetd(void* Dst,int Val,unsigned Bytes);
   
   unsigned PeekW(unsigned Memory);
   
   //Complex Functions
   void nyloxMemCopy(const void *Src,void *Dest,unsigned Bytes);
   void nyloxMemSet(void *Dest,unsigned char Value,unsigned Bytes);
   void nyloxMemClear(void *Dest,unsigned Bytes);

   
  /** Heap References **/
  
  #define HEAP_INDEX_SIZE   0x20000
  #define HEAP_MAGIC        0x123890AB
  #define HEAP_MIN_SIZE     0x70000

  typedef struct {
   DWORD  Magic;
   BYTE   Hole;
   DWORD  Size;
  }heap_Header_T;

  typedef struct {
   DWORD  Magic; 
   heap_Header_T* Header;
  }heap_Footer_T;

  typedef struct {
   BYTE   Supervisor : 1;
   BYTE   ReadOnly   : 1;
   BYTE   WriteOnly  : 1;
   BYTE   Reserved   : 5;
  }heap_Property_T;

  typedef struct {
   ordered_array_t Index;
   
   DWORD  		 StartAddr;
   DWORD  		 EndAddr;
   DWORD  		 MaxAddr;
   
   BYTE			 Supervisor;
   BYTE			 ReadOnly;
   
  }heap_General_T;

  
   heap_General_T *CreateHeap(DWORD Start, DWORD End, DWORD Max, BYTE Supervisor, BYTE ReadOnly);
   void* HeapAlloc(DWORD Size, BYTE PageAlign, heap_General_T *Heap);
   void  HeapFree(void *P, heap_General_T *Heap); 
   
#define _libcmemory_H
#endif

