//
//  Nylox
//  Copyright (C) 2006-2008 Agustin Alvarez
// 
//  This program is free software; you can redistribute it and/or modify it
//  under the terms of the GNU General Public License as published by the Free
//  Software Foundation; either version 2 of the License, or (at your option)
//  any later version.
// 
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
//  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
//  for more details.
//  
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//  Memory.c
//
// Physical Memory Includes:
//				    Best Fits ->
//				    Contraction
//				    Optimizer
//
// TODO: Physical Memory Change KMalloc With A Heap :)

#include <Arch/IA32/Memory.h>
#include <Arch/IA32/Hal.h>

// Macros used in the bitset algorithms.
#define INDEX_FROM_BIT(a) (a/(8*4))
#define OFFSET_FROM_BIT(a) (a%(8*4))
#define MAX_PAGES(a,b) (a/b/32)

#define KHEAP_START         0xC0000000
#define KHEAP_INITIAL_SIZE  0x100000

DWORD*			   PhysicalMem;
DWORD			   Physical;
DWORD			   PhysicalSize;

Page_Directory_T*  Kernel_Directory;
Page_Directory_T*  Current_Directory;
DWORD* 			   pFrames;
DWORD  			   lFrames;

heap_General_T	   *KHeap;
 
extern 			   DWORD Placement_Address;


void mm_Arch_PhysicalInit(DWORD ExtendedMemory,DWORD KernelStart,DWORD KernelEnd)
{
 Physical = ExtendedMemory / 0x1000;
 PhysicalSize = ExtendedMemory / 1000;
 
 Printf("%dKB Physical Memory\n",PhysicalSize);
 
 /** Create Or  Bitmap **/
 pFrames = (DWORD*)KMalloc(INDEX_FROM_BIT(Physical)); 
 /** Set All Pages Free **/
 Memsetd(PhysicalMem,0,INDEX_FROM_BIT(Physical));
 /** Mark Reserved Sections **/
 DWORD L;
	/** BIOS And Video DATA **/
	for(L=0xA0000;L<0x100000;L+=4096);
	   mm_Arch_PhysicalAllocPage(L/4096);
	/** Kernel Now **/
	for(L=KernelStart;L<KernelEnd;L += 4096);
	   mm_Arch_PhysicalAllocPage(L/4096);
	   
	Printf("%d Total, %d Free Pages\n",Physical,(Physical-(((0xA0000+0x100000)/4096)+((KernelStart+KernelEnd)/4096))));

}

/** Free A Page **/
void mm_Arch_PhysicalFreePage(DWORD PageN)
{
 if(!Test_Frame(PhysicalMem,PageN)) {
	Printf("Warning: Page %d, Its Already Allocated..\n",PageN);
 }
 
 Clear_Frame(PhysicalMem,PageN);
 
 return;
}

/** Allocate A Page **/
void mm_Arch_PhysicalAllocPage(DWORD PageN)
{
 if(Test_Frame(PhysicalMem,PageN)) {
	Printf("Warning: Page %d, Is Already Allocated..\n",PageN);
 }
 
 Set_Frame(PhysicalMem,PageN);
 
 return;
}
/** If A Page Its Free? **/
BYTE mm_Arch_PhysicalIsPageFree(DWORD PageN)
{
 return !Test_Frame(PhysicalMem,PageN);
}

/** Find A Free Page **/
DWORD mm_Arch_PhysicalFindFreePage(DWORD Start)
{
 DWORD L;
 DWORD Idx = Start/4096;
 
 for(L = Idx;L<Physical;L++) {
	 if(mm_Arch_PhysicalIsPageFree(L)) {
		return L*4096;
	 }
 }
 return (DWORD)-1;
}

/** Find A Range Of Free Page **/
DWORD mm_Arch_PhysicalFindFreeRangePage(DWORD Start,DWORD Size)
{
 DWORD S = Start;
 DWORD F = 0;

 while(S != -1) {
	 S = mm_Arch_PhysicalFindFreePage(S);
	 F = mm_Arch_PhysicalFindFreePage(S);
	 if((F-S) <= 0) { /** Found A Clear Match :) **/
		return (S);
	 }
 }
 return (DWORD)-1;
}

/** -------------------------------------------- VIRTUAL -------------------------------------------- **/
void mm_Arch_VirtualInit(DWORD ExtendedMemory)
{

 lFrames = ExtendedMemory / 0x1000;
 pFrames = (DWORD*)KMalloc(INDEX_FROM_BIT(lFrames));
 Memsetb(pFrames,0,INDEX_FROM_BIT(lFrames));
 
 /** Lets Make A Page Directory **/
 Kernel_Directory  = (Page_Directory_T*)KMalloc_A(sizeof(Page_Directory_T));
 Memsetb(Kernel_Directory,0,sizeof(Page_Directory_T));
 Current_Directory = Kernel_Directory;
   
 /** We need to identity map (phys addr = virt addr) from 0x0 to the end of used memory, so we can access this
         transparently, as if paging wasn't enabled. NOTE that we use a while loop here deliberately.
	inside the loop body we actually change placement_address by calling kmalloc(). A while loop causes this to be
	computed on-the-fly rather than once at the start **/
 int I = 0;

 /** Map some pages in the kernel heap area**/
 for (I  = KHEAP_START; I  < KHEAP_START+KHEAP_INITIAL_SIZE; I  += 0x1000)
   mm_Arch_VirtualGetPage(I , 1, Kernel_Directory); 
	
 /** Map Kernel Memory **/
 I = 0;
 
 while(I < Placement_Address+0x1000 ) {
  mm_Arch_VirtualAllocFrame(mm_Arch_VirtualGetPage(I,1,Kernel_Directory),0,0); // Kernel code is readable but not writeable from userspace.
  I += 0x1000;
 }

 /** Map some pages in the kernel heap area**/
 for (I  = KHEAP_START; I  < KHEAP_START+KHEAP_INITIAL_SIZE; I  += 0x1000)
    mm_Arch_VirtualAllocFrame(mm_Arch_VirtualGetPage(I , 1, Kernel_Directory),0,0); 
 
 /** Set Our Interrumpt **/
 HALInterruptSet(14,I86_IDT_DESC_PRESENT | I86_IDT_DESC_BIT32,0x8,&mm_Arch_VirtualPageFault);
 
 /** Enable Paging **/
 mm_Arch_SwitchPageDirectory(Kernel_Directory);
 

 /** Initialise the kernel heap. **/
 KHeap = CreateHeap(KHEAP_START, KHEAP_START+KHEAP_INITIAL_SIZE, 0xCFFFF000, 0, 0); 
 
}

void mm_Arch_SwitchPageDirectory(Page_Directory_T* Dir)
{
 DWORD CR0;
 
 Current_Directory = Dir; // For Know What Its The Current Dir

 kernelProcessorSetCR3(&Dir->TablesPhy);
 kernelProcessorGetCR0(CR0);
 CR0 |= 0x80000000; // Wiii, We Enabled Paging At Last!
 kernelProcessorSetCR0(CR0);

}

Page_T* mm_Arch_VirtualGetPage(DWORD Address,int Make,Page_Directory_T *Dir)
{
 Address /= 0x1000;
 
 DWORD TableIdx = Address / 1024;
 
 
 if(Dir->Tables[TableIdx]) { /** The Page Its Already Assign it **/
   return &Dir->Tables[TableIdx]->Pages[Address % 1024];
 }
 else if(Make)
 {
   DWORD Tmp;
   
   Dir->Tables[TableIdx] = (Page_Table_T*)KMalloc_AP(sizeof(Page_Table_T),&Tmp);
   Memsetb(Dir->Tables[TableIdx],0,0x1000);
   
   Dir->TablesPhy[TableIdx] = Tmp | 0x7; // PRESENT, READWRITE, USER

   return &Dir->Tables[TableIdx]->Pages[Address % 1024];
 }
 else {
   return 0;
 }
}

void mm_Arch_VirtualAllocFrame(Page_T* Page,int PVL,int RW)
{
 if(Page->Frame != 0) {
   return;
 }
 else {
   DWORD Idx = First_Frame(pFrames,lFrames);
   if(Idx == (DWORD)-1) return; 		/** Make A Panic **/
   Set_Frame(pFrames,Idx*0x1000);		/** This Frame Is Ours **/
   Page->Present = 0x1;					/** Mark As Present **/
   Page->RW = (RW)?1:0;					/** Writeable or Read Only **/
   Page->User = (PVL)?0:1;				/** System or User? **/
   Page->Frame = Idx;
 }
}
 
void mm_Arch_VirtualFreeFrame(Page_T* Page)
{
 DWORD Frame;

 if(!(Frame = Page->Frame))
   return;
 else {
   Clear_Frame(pFrames,Frame);
   Page->Frame = 0x0;
 }
}


void mm_Arch_VirtualPageFault(TRAP_FRAME TF)
{
 DWORD Addr;
 
 int Present = !(TF.ERR & 0x1);		// Page not present
 int RW 	 = (TF.ERR & 0x2);		// Write operation?
 int US 	 = (TF.ERR & 0x4);		// Processor was in user-mode?
 int RSV	 = (TF.ERR & 0x8);		// Overwritten CPU-reserved bits of page entry?
 //int ID		 = !(TF->ERR & 0x10);		// Caused by an instruction fetch?
 
 Printf(" CPU Page Fault ( ");
 if(Present) Printf("Present ");
 if(RW) Printf("Read-Only ");
 if(US) Printf("User-Mode ");
 if(RSV) Printf("Reserved ");
 
 kernelProcessorGetCR2(Addr);
 
 Printf(") AT 0x%X",Addr);
 
 kernelProcessorStop();
}

void* mm_Arch_Malloc(DWORD Size,BYTE Align,DWORD* Phys)
{
 void* Addr = HeapAlloc(Size,Align,KHeap);
 

 if(Phys != 0x0) {
    Page_T* Page = mm_Arch_VirtualGetPage((DWORD)Addr,0x0,Kernel_Directory);
	*Phys = (Page->Frame*0x1000) + ((DWORD)Addr&0xFFF);
 }
 return Addr;
}

void  mm_Arch_Free(void* P)
{
 HeapFree(P,KHeap);
}