//
//  Nylox
//  Copyright (C) 2006-2008 Agustin Alvarez
// 
//  This program is free software; you can redistribute it and/or modify it
//  under the terms of the GNU General Public License as published by the Free
//  Software Foundation; either version 2 of the License, or (at your option)
//  any later version.
// 
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
//  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
//  for more details.
//  
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


#include <Arch/IA32/Hal.h>

#include <Libc.h>
#include <tty.h>
#include <exception.h>
#include <Schedule.h>

/** Kernel Built In Exceptions **/
extern void TRAP_00(),TRAP_01(),TRAP_02(),TRAP_03(),TRAP_04(),TRAP_05(),TRAP_06(),TRAP_07(),
			TRAP_08(),TRAP_09(),TRAP_0A(),TRAP_0B(),TRAP_0C(),TRAP_0D(),TRAP_0E(),TRAP_0F(),
			TRAP_10(),TRAP_11(),TRAP_12(),TRAP_13();

char szExceptionType[20][50]=
{
    "Divide by zero",
    "Debug",
    "NMI",
    "Breakpoint",
    "Overflow",
    "Bound range exceeded",
    "Invalide opcode",
    "Device not available",
    "Double fault",
    "Coprocessor segment overrun",
    "Invalid TSS",
    "Segment not present",
    "Stack fault",
    "General protection fault",
    "Page fault",
    "Floating point error",
    "Alignment check",
    "Machine check",
    "SIMD Floating point",
    "Unknown Exception",
};

void ExceptionHandle(TRAP_FRAME TF)
{ 
 if(!(TF.CS & 0x3)) {
	kernelConsole_color(kernelConsole_current(),((COLOR_BLUE << 12) | ((COLOR_WHITE & 0x0F) << 8)));
	kernelConsole_clear(kernelConsole_current());
	
	Printf("=======================================\n");
	Printf(">> Nylox <<\n");
	Printf(">> Kernel PaNIC! << %s\n",szExceptionType[TF.TRAPN]);
	Printf("=======================================\n");
	Printf("EAX=%08X EBX=%08X\nECX=%08X EDX=%08X\nESI=%08X EDI=%08X\n",TF.EAX,TF.EBX,TF.ECX,TF.EDX,TF.ESI,TF.EDI);
	Printf("CS=%04X      DS=%04X\nES=%04X      FS=%04X\nGS=%04X      SS=%04X\nEIP=%08X\n",TF.CS,TF.DS,TF.ES,TF.FS,TF.GS,TF.SS,TF.EIP);
	Printf("Flags=%08X\n",TF.EFLAGS);
	
	kernelProcessorStop();
 }
 
}

void nyloxInterrumptSet()
{
  /** Set x86-32 Interrumpts **/
  HALInterruptSet(0,I86_IDT_DESC_PRESENT | I86_IDT_DESC_BIT32,0x8,&TRAP_00);
  HALInterruptSet(1,I86_IDT_DESC_PRESENT | I86_IDT_DESC_BIT32,0x8,&TRAP_01);
  HALInterruptSet(2,I86_IDT_DESC_PRESENT | I86_IDT_DESC_BIT32,0x8,&TRAP_02);
  HALInterruptSet(3,I86_IDT_DESC_PRESENT | I86_IDT_DESC_BIT32,0x8,&TRAP_03);
  HALInterruptSet(4,I86_IDT_DESC_PRESENT | I86_IDT_DESC_BIT32,0x8,&TRAP_04);
  HALInterruptSet(5,I86_IDT_DESC_PRESENT | I86_IDT_DESC_BIT32,0x8,&TRAP_05);
  HALInterruptSet(6,I86_IDT_DESC_PRESENT | I86_IDT_DESC_BIT32,0x8,&TRAP_06);
  HALInterruptSet(7,I86_IDT_DESC_PRESENT | I86_IDT_DESC_BIT32,0x8,&TRAP_07);
  HALInterruptSet(8,I86_IDT_DESC_PRESENT | I86_IDT_DESC_BIT32,0x8,&TRAP_08);
  HALInterruptSet(9,I86_IDT_DESC_PRESENT | I86_IDT_DESC_BIT32,0x8,&TRAP_09);
  HALInterruptSet(10,I86_IDT_DESC_PRESENT | I86_IDT_DESC_BIT32,0x8,&TRAP_0A);
  HALInterruptSet(11,I86_IDT_DESC_PRESENT | I86_IDT_DESC_BIT32,0x8,&TRAP_0B);
  HALInterruptSet(12,I86_IDT_DESC_PRESENT | I86_IDT_DESC_BIT32,0x8,&TRAP_0C);
  HALInterruptSet(13,I86_IDT_DESC_PRESENT | I86_IDT_DESC_BIT32,0x8,&TRAP_0D);
  HALInterruptSet(14,I86_IDT_DESC_PRESENT | I86_IDT_DESC_BIT32,0x8,&TRAP_0E);
  HALInterruptSet(15,I86_IDT_DESC_PRESENT | I86_IDT_DESC_BIT32,0x8,&TRAP_0F);
  HALInterruptSet(16,I86_IDT_DESC_PRESENT | I86_IDT_DESC_BIT32,0x8,&TRAP_10);
  HALInterruptSet(17,I86_IDT_DESC_PRESENT | I86_IDT_DESC_BIT32,0x8,&TRAP_11);
  HALInterruptSet(18,I86_IDT_DESC_PRESENT | I86_IDT_DESC_BIT32,0x8,&TRAP_12);
  HALInterruptSet(19,I86_IDT_DESC_PRESENT | I86_IDT_DESC_BIT32,0x8,&TRAP_13);
  
  /** Set Reserved Interrumpts **/
  BYTE I = 20;
  for(;I<32;I++) HALInterruptSet(I,I86_IDT_DESC_PRESENT | I86_IDT_DESC_BIT32,0x8,&TRAP_0F);
  
}
