//
//  Nylox
//  Copyright (C) 2006-2008 Agustin Alvarez
// 
//  This program is free software; you can redistribute it and/or modify it
//  under the terms of the GNU General Public License as published by the Free
//  Software Foundation; either version 2 of the License, or (at your option)
//  any later version.
// 
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
//  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
//  for more details.
//  
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
// K32Keyboard.c
//
// Nativa Driver Module

#include <Arch/IA32/Hal.h>

#include <Libc.h>
#include <Keyboard.h>

/* "bucky bits" or "meta" keys
0x0100 is reserved for non-ASCII keys, so start with 0x200 */
#define	KBD_META_ALT	0x0200	/* Alt is pressed */
#define	KBD_META_CTRL	0x0400	/* Ctrl is pressed */
#define	KBD_META_SHIFT	0x0800	/* Shift is pressed */
#define	KBD_META_ANY	(KBD_META_ALT | KBD_META_CTRL | KBD_META_SHIFT)
#define	KBD_META_CAPS	0x1000	/* CapsLock is on */
#define	KBD_META_NUM	0x2000	/* NumLock is on */
#define	KBD_META_SCRL	0x4000	/* ScrollLock is on */

/* "raw" set 1 scancodes from PC keyboard. Keyboard info here:
http://my.execpc.com/~geezer/osd/kbd */
#define	RAW1_LEFT_CTRL		0x1D
#define	RAW1_RIGHT_CTRL		0x1D	/* same as left */
#define	RAW1_LEFT_SHIFT		0x2A
#define	RAW1_RIGHT_SHIFT	0x36
#define	RAW1_LEFT_ALT		0x38
#define	RAW1_RIGHT_ALT		0x38	/* same as left */
#define	RAW1_CAPS_LOCK		0x3A
#define	RAW1_F1			0x3B
#define	RAW1_F2			0x3C
#define	RAW1_F3			0x3D
#define	RAW1_F4			0x3E
#define	RAW1_F5			0x3F
#define	RAW1_F6			0x40
#define	RAW1_F7			0x41
#define	RAW1_F8			0x42
#define	RAW1_F9			0x43
#define	RAW1_F10		0x44
#define	RAW1_NUM_LOCK		0x45
#define	RAW1_SCROLL_LOCK	0x46
#define	RAW1_DEL		0x53
#define	RAW1_F11		0x57
#define	RAW1_F12		0x58

/*function keys: */
#define	KEY_F1		0x80
#define	KEY_F2		(KEY_F1 + 1)
#define	KEY_F3		(KEY_F2 + 1)
#define	KEY_F4		(KEY_F3 + 1)
#define	KEY_F5		(KEY_F4 + 1)
#define	KEY_F6		(KEY_F5 + 1)
#define	KEY_F7		(KEY_F6 + 1)
#define	KEY_F8		(KEY_F7 + 1)
#define	KEY_F9		(KEY_F8 + 1)
#define	KEY_F10		(KEY_F9 + 1)
#define	KEY_F11		(KEY_F10 + 1)
#define	KEY_F12		(KEY_F11 + 1)
/* cursor keys */
#define	KEY_INS		0x90
#define	KEY_DEL		(KEY_INS + 1)
#define	KEY_HOME	(KEY_DEL + 1)
#define	KEY_END		(KEY_HOME + 1)
#define	KEY_PGUP	(KEY_END + 1)
#define	KEY_PGDN	(KEY_PGUP + 1)
#define	KEY_LEFT	(KEY_PGDN + 1)	/* esc[D */
#define	KEY_UP		(KEY_LEFT + 1)	/* esc[A */
#define	KEY_DOWN	(KEY_UP + 1)	/* esc[B */
#define	KEY_RIGHT	(KEY_DOWN + 1)	/* esc[C */
/* print screen/sys rq and pause/break */
#define	KEY_PRNT	(KEY_RIGHT + 1)
#define	KEY_PAUSE	(KEY_PRNT + 1)
/* these return a value but they could also act as additional bucky keys */
#define	KEY_LWIN	(KEY_PAUSE + 1)
#define	KEY_RWIN	(KEY_LWIN + 1)
#define	KEY_MENU	(KEY_RWIN + 1)


static unsigned char KeyboardBuffer[KEYBOARD_BUFFER_SIZE];  //Buffer
static BYTE			 KeyboardLast;							//Track
static BYTE			 KeyboardInit;							//Initialized

void KeyboardIsr();
void KeyboardIrq();

asm (
   ".globl _KeyboardIsr     \n"
   "_KeyboardIsr:           \n"
   "   pusha               \n"
   "   pushw %ds           \n"
   "   pushw %es           \n"
   "   pushw %ss           \n" 
   "   pushw %ss           \n"
   "   popw %ds            \n"
   "   popw %es            \n"
   "                       \n"
   "   call _KeyboardIrq    \n"
   "                       \n"
   "   popw %es            \n"
   "   popw %ds            \n"
   "   popa                \n"
   "   iret                \n" 
);


void KeyboardIrq()
{
 unsigned char KeyCode;

 /** Say To Hal, We Are Done **/
 HALIrqDone(0x1);
 
 /** Get Keystroke Press **/
 kernelProcessorInPort8(0x60,KeyCode);
 
 /** We Dont Process The Key, If The Buffer ITs Full **/
 if(KeyboardLast >= KEYBOARD_BUFFER_SIZE) { return; }

 KeyboardBuffer[KeyboardLast] = KeyCode;
 KeyboardLast++;

}


int Keyboard_Install()
{
 nyloxMemClear(&KeyboardBuffer,sizeof(unsigned char)*KEYBOARD_BUFFER_SIZE);
 KeyboardLast = 0;
 KeyboardInit = 1;
 
 HALInterruptSet(33,I86_IDT_DESC_PRESENT | I86_IDT_DESC_BIT32,0x8,&KeyboardIsr);
 HALIrqEnable(1);
 
 return KeyboardInit;
}

void Keyboard_Write(BYTE Addr,BYTE Data)
{
 unsigned long Timeout;
 BYTE Stat;
 
 for(Timeout = 500000L; Timeout != 0; Timeout--)
 {
  kernelProcessorInPort8(0x64,Stat);
  if((Stat & 0x2) == 0)
    break;
 }
 
 if(Timeout != 0)
   kernelProcessorOutPort8(Addr,Data);
}
 

/*** Native Convert Table ***/
unsigned char Keyboard_Convert(unsigned char Key)
{
	static const unsigned char set1_map[] =
	{
/* 00 */0,	0x1B,	'1',	'2',	'3',	'4',	'5',	'6',
/* 08 */'7',	'8',	'9',	'0',	'-',	'=',	'\b',	'\t',
/* 10 */'q',	'w',	'e',	'r',	't',	'y',	'u',	'i',
/* 1Dh is left Ctrl */
/* 18 */'o',	'p',	'[',	']',	'\n',	0,	'a',	's',
/* 20 */'d',	'f',	'g',	'h',	'j',	'k',	'l',	';',
/* 2Ah is left Shift */
/* 28 */'\'',	'`',	0,	'\\',	'z',	'x',	'c',	'v',
/* 36h is right Shift */
/* 30 */'b',	'n',	'm',	',',	'.',	'/',	0,	0,
/* 38h is left Alt, 3Ah is Caps Lock */
/* 38 */0,	' ',	0,	KEY_F1,	KEY_F2,	KEY_F3,	KEY_F4,	KEY_F5,
/* 45h is Num Lock, 46h is Scroll Lock */
/* 40 */KEY_F6,	KEY_F7,	KEY_F8,	KEY_F9,	KEY_F10,0,	0,	KEY_HOME,
/* 48 */KEY_UP,	KEY_PGUP,'-',	KEY_LEFT,'5',	KEY_RIGHT,'+',	KEY_END,
/* 50 */KEY_DOWN,KEY_PGDN,KEY_INS,KEY_DEL,0,	0,	0,	KEY_F11,
/* 58 */KEY_F12
	};
	static const unsigned char shift1_map[]=
	{
/* 00 */0,	0x1B,	'!',	'@',	'#',	'$',	'%',	'^',
/* 08 */'&',	'*',	'(',	')',	'_',	'+',	'\b',	'\t',
/* 10 */'Q',	'W',	'E',	'R',	'T',	'Y',	'U',	'I',
/* 1Dh is left Ctrl */
/* 18 */'O',	'P',	'{',	'}',	'\n',	0,	'A',	'S',
/* 20 */'D',	'F',	'G',	'H',	'J',	'K',	'L',	':',
/* 2Ah is left Shift */
/* 28 */'\"',	'~',	0,	'|',	'Z',	'X',	'C',	'V',
/* 36h is right Shift */
/* 30 */'B',	'N',	'M',	'<',	'>',	'?',	0,	0,
/* 38h is left Alt, 3Ah is Caps Lock */
/* 38 */0,	' ',	0,	KEY_F1,	KEY_F2,	KEY_F3,	KEY_F4,	KEY_F5,
/* 45h is Num Lock, 46h is Scroll Lock */
/* 40 */KEY_F6,	KEY_F7,	KEY_F8,	KEY_F9,	KEY_F10,0,	0,	KEY_HOME,
/* 48 */KEY_UP,	KEY_PGUP,'-',	KEY_LEFT,'5',	KEY_RIGHT,'+',	KEY_END,
/* 50 */KEY_DOWN,KEY_PGDN,KEY_INS,KEY_DEL,0,	0,	0,	KEY_F11,
/* 58 */KEY_F12
	};
	
	BYTE KeyStatus;
	unsigned I;
	
	if(Key >= 0x80)
	{
	 Key &= 0x7F;
	 
	 if(Key == RAW1_LEFT_ALT || Key == RAW1_RIGHT_ALT)
			KeyStatus &= ~KBD_META_ALT;
	 else if(Key == RAW1_LEFT_CTRL || Key == RAW1_RIGHT_CTRL)
			KeyStatus &= ~KBD_META_CTRL;
	 else if(Key == RAW1_LEFT_SHIFT || Key == RAW1_RIGHT_SHIFT)
			KeyStatus &= ~KBD_META_SHIFT;
	 return 0;
	}
/* it's a make key (key preseed): check the "meta" keys, as above */
	if(Key == RAW1_LEFT_ALT || Key == RAW1_RIGHT_ALT)
	{
		KeyStatus |= KBD_META_ALT;
		return 0;
	}
	if(Key == RAW1_LEFT_CTRL || Key == RAW1_RIGHT_CTRL)
	{
		KeyStatus |= KBD_META_CTRL;
		return 0;
	}
	if(Key == RAW1_LEFT_SHIFT || Key == RAW1_RIGHT_SHIFT)
	{
		KeyStatus |= KBD_META_SHIFT;
	    return 0;
	}
/* Scroll Lock, Num Lock, and Caps Lock set the LEDs. These keys
have on-off (toggle or XOR) action, instead of momentary action */
	if(Key == RAW1_SCROLL_LOCK)
	{
		KeyStatus ^= KBD_META_SCRL;
		goto LEDS;
	}
	if(Key == RAW1_NUM_LOCK)
	{
		KeyStatus ^= KBD_META_NUM;
		goto LEDS;
	}
	if(Key == RAW1_CAPS_LOCK)
	{
		KeyStatus ^= KBD_META_CAPS;
LEDS:	
		Keyboard_Write(0x60, 0xED);			/* "set LEDs" command */
		I = 0;
		if(KeyStatus & KBD_META_SCRL)
			I |= 1;
		if(KeyStatus & KBD_META_NUM)
			I |= 2;
		if(KeyStatus & KBD_META_CAPS)
			I |= 4;
		Keyboard_Write(0x60,I );				/* bottom 3 bits set LEDs */
		return 0;
	}
/* now that we've tested for CTRL and ALT,
we can handle the three-finger salute */
	if((KeyStatus & KBD_META_CTRL) &&
		(KeyStatus & KBD_META_ALT) && Key == RAW1_DEL)
	{
		Printf("Crtl + Alt + Del Pressed");
	}
/* ignore invalid scan codes */
	if(Key >= sizeof(set1_map) / sizeof(set1_map[0]))
		return 0;
/* handle Ctrl+A-Z */
	if(KeyStatus & KBD_META_CTRL)
	{
		I = set1_map[Key];
		if(I < 'a' || I > 'z')
			return 0;
		return I - 'a' + 1;
	}
/* handle Shift */
	if(KeyStatus & KBD_META_SHIFT)
		I = shift1_map[Key];
	else
		I = set1_map[Key];
	
	return I;
}


unsigned char Keyboard_GetKey()
{
 unsigned char rET;
 
 /** If The Buffer its Not Empty **/
 if(KeyboardLast > 0)
 {
  KeyboardLast--;					  //One Key Less
  rET = KeyboardBuffer[KeyboardLast]; //Get The Key
  KeyboardBuffer[KeyboardLast] = 0;	  //We Clear IT
 }
 return Keyboard_Convert(rET);
}

