//
//  Nylox
//  Copyright (C) 2006-2008 Agustin Alvarez
// 
//  This program is free software; you can redistribute it and/or modify it
//  under the terms of the GNU General Public License as published by the Free
//  Software Foundation; either version 2 of the License, or (at your option)
//  any later version.
// 
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
//  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
//  for more details.
//  
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


#include <Boot.h>
#include <Libc.h>
#include <Keyboard.h>
#include <Exception.h>

/** Arch Includes **/
#include <Arch/IA32/Hal.h>
#include <Memory.h>
#include <tty.h>
#include <Schedule.h>


#define KERNEL_ERR_INITIALIZE -2
#define NYLOX_MVERSION 0
#define NYLOX_SVERSION 0
#define NYLOX_RVERSION 20

extern DWORD  Placement_Address;
extern DWORD  end;

kernelConsole_tty *VC[2];
 
int nyloxInitialize(Multiboot_Header_T *Header, Multiboot_Info_T *Info)
{
 /** Disable Interrumpts **/
 HalDisableInterrumpt();		
 
 /** Initialize KMalloc Function **/
 Placement_Address = (Header->Bss_end_addr & 0xFFFFF000) + 0x1000;
 
 /**Initialize Hal Device **/
 if(HALInitialize() == 0x0) return KERNEL_ERR_INITIALIZE;
 
 /** Initialize The Exceptions **/
 nyloxInterrumptSet();

 /** Enable Interrumpts (God Save Nylox) **/
 HalEnableInterrumpt();
 
 /** Initialize Memory Manager (Physical And Virtual) **/
 mmPhysicalInit(Info->Mem_Upper*1000,Header->Entry_addr,(DWORD)&end);
 mmVirtualInit(Info->Mem_Upper*1000);

 /** Initialize Virtual Consoles **/
 VC[0] = kernelConsole_init(80,25,2,0x0F00,1);
 VC[1] = kernelConsole_init(80,25,2,0x0F00,1);
 VC[2] = kernelConsole_init(80,25,2,0x0F00,1);
 kernelConsole_cursor(VC[0],0x0);
 kernelConsole_draw(VC[0]); //Virtual Console 1
 
 kernelConsole_putstr(VC[0],"------ Virtual Console 0 ------\n");
 kernelConsole_putstr(VC[1],"------ Virtual Console 1 ------\n");
 kernelConsole_putstr(VC[2],"------ Virtual Console 2 ------\n");
 Printf("* Initializing Nylox v%d.%d.%d\n",NYLOX_MVERSION,NYLOX_SVERSION,NYLOX_RVERSION);

 /** Initialize Keyboard **/
 if(Keyboard_Install() == 0x0) return KERNEL_ERR_INITIALIZE;
 Printf("* Keyboard Installed\n");
 
 /** Initialize Schedule **/
 sched_Init();
  
 return 0x1;
}

void Shell()
{
 int V = 0;
 char* Buffer = (char*)mmMalloc(512,0,0);
 int Size=0;
  
 for(;;) {
	Printf("Nylox > ");
    Size = kernelConsole_waitkey(VC[V],Buffer,10,1);
	Printf("\n");
	
	if(Strcmp(Buffer,"vc0")) {
	  V = 0;
	  kernelConsole_draw(VC[V]);
	}else if(Strcmp(Buffer,"vc1")) {
	  V = 1;
	  kernelConsole_draw(VC[V]);
	}else if(Strcmp(Buffer,"vc2")) {
	  V = 2;
	  kernelConsole_draw(VC[V]);
	}
	
	Memsetb((BYTE*)Buffer,0x0,Size);
	
 }

}

 
int KMain(Multiboot_Header_T *Header, Multiboot_Info_T *Info)
{
 
 if(nyloxInitialize(Header,Info) == KERNEL_ERR_INITIALIZE)
   return 0;
 
 Shell();
 

 return 0;
}