;;
;;  Nylox
;;  Copyright (C) 2006-2008 Agustin Alvarez
;; 
;;  This program is free software; you can redistribute it and/or modify it
;;  under the terms of the GNU General Public License as published by the Free
;;  Software Foundation; either version 2 of the License, or (at your option)
;;  any later version.
;; 
;;  This program is distributed in the hope that it will be useful, but
;;  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
;;  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
;;  for more details.
;;  
;;  You should have received a copy of the GNU General Public License along
;;  with this program; if not, write to the Free Software Foundation, Inc.,
;;  59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
;;
;;  Entry.asm
;;

[SECTION .text]

		 [GLOBAL Entry]
		 EXTERN bss, end
		 EXTERN _KMain
		 
		DS_MAGIC		equ	3544DA2Ah

		MULTIBOOT_PAGE_ALIGN	equ 1<<0
		MULTIBOOT_MEMORY_INFO	equ 1<<1
		MULTIBOOT_AOUT_KLUDGE	equ 1<<16
		MULTIBOOT_HEADER_MAGIC	equ 0x1BADB002
		MULTIBOOT_HEADER_FLAGS	equ MULTIBOOT_PAGE_ALIGN | MULTIBOOT_MEMORY_INFO | MULTIBOOT_AOUT_KLUDGE
		MULTIBOOT_CHECKSUM	equ -(MULTIBOOT_HEADER_MAGIC + MULTIBOOT_HEADER_FLAGS)

Entry:
	; check if data segment linked, located, and loaded properly
	MOV EAX,[ds_magic]
	CMP EAX,DS_MAGIC
	JE  DS_OK

; display a blinking white-on-blue 'D' and freeze
	MOV WORD [0xB8000],0x9F44
	JMP SHORT $
DS_OK:

	; zero the C language BSS
	; 'bss' and 'end' are defined in the linker script file
	MOV EDI,bss
	MOV ECX,end
	SUB ECX,EDI
	XOR EAX,EAX
	REP STOSB

	MOV ESP,stack

	MOV DWORD[lpMultiBootInfo],EBX	;GRUB Information

	; GRUB 0.90 leaves the NT bit set in EFLAGS. The first IRET we attempt
	; will cause a TSS-based task-switch, which will cause Exception 10.
	; Let's prevent that:
	PUSH DWORD 0x2
	POPF

	PUSH DWORD [lpMultiBootInfo]
	PUSH DWORD mboot
	CALL _KMain			; call C code
	
	JMP  $				; freeze

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Multiboot header for GRUB bootloader. This must be in the first 8K
; of the kernel file. We use the aout kludge so it works with ELF,
; DJGPP COFF, Win32 PE, or other formats.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; these are in the linker script file
EXTERN code, bss, end

ALIGN 4
mboot:
	dd MULTIBOOT_HEADER_MAGIC
	dd MULTIBOOT_HEADER_FLAGS
	dd MULTIBOOT_CHECKSUM
; aout kludge. These must be PHYSICAL addresses
	dd mboot
	dd code
	dd bss
	dd end
	dd Entry

; ============ EXCEPTIONS ========================
%IMACRO DO_TRAP_NO_ERR 2
GLOBAL _TRAP_%1
align 4
_TRAP_%1: \
		PUSH DWORD 0
        PUSH DWORD %2
        JMP  ALL_TRAPS
%ENDMACRO

%IMACRO DO_TRAP_ERR 2
GLOBAL _TRAP_%1
align 4
_TRAP_%1:
		PUSH DWORD %2
        JMP	 ALL_TRAPS
%ENDMACRO

DO_TRAP_NO_ERR 00, 0
DO_TRAP_NO_ERR 01, 1
DO_TRAP_NO_ERR 02, 2
DO_TRAP_NO_ERR 03, 3
DO_TRAP_NO_ERR 04, 4
DO_TRAP_NO_ERR 05, 5
DO_TRAP_NO_ERR 06, 6
DO_TRAP_NO_ERR 07, 7
DO_TRAP_NO_ERR 08, 8
DO_TRAP_NO_ERR 09, 9
DO_TRAP_ERR    0A, 10
DO_TRAP_ERR    0B, 11
DO_TRAP_ERR    0C, 12
DO_TRAP_ERR    0D, 13
DO_TRAP_ERR    0E, 14
DO_TRAP_NO_ERR 0F, 15
DO_TRAP_NO_ERR 10, 16
DO_TRAP_NO_ERR 11, 17
DO_TRAP_NO_ERR 12, 18
DO_TRAP_NO_ERR 13, 19
DO_TRAP_NO_ERR 14, 104		;Schedule

EXTERN _ExceptionHandle
align 4
ALL_TRAPS:
	PUSHA
    
	MOV     AX, DS
	PUSH    EAX
    MOV     AX, ES
	PUSH    EAX
	MOV     AX, FS
	PUSH    EAX
	MOV     AX, GS
	PUSH    EAX
	
	MOV 	AX, 0x10
    MOV		DS, AX
    MOV		ES, AX
	MOV     FS, AX
	MOV     GS, AX
	
    CALL	_ExceptionHandle
	
    POP		EAX
	
	MOV		DS, AX
	MOV     ES, AX
	MOV     FS, AX
	MOV     GS, AX
	
    POPA
    ADD		ESP, 0x8
	STI
    IRET
	
[SECTION .data]
  lpMultiBootInfo DD 0x0
  ds_magic 		  DD DS_MAGIC

[SECTION .bss]
	RESD 1024
stack:
