//
//  Nylox
//  Copyright (C) 2006-2008 Agustin Alvarez
// 
//  This program is free software; you can redistribute it and/or modify it
//  under the terms of the GNU General Public License as published by the Free
//  Software Foundation; either version 2 of the License, or (at your option)
//  any later version.
// 
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
//  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
//  for more details.
//  
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include <Memory.h>
#include <tty.h>
#include <Keyboard.h>

kernelConsole_tty* Current;

kernelConsole_tty *kernelConsole_init(BYTE X,BYTE Y,BYTE BPP,WORD COLOR,BYTE SCROLL)
{
 kernelConsole_tty* CON = (kernelConsole_tty*)mmMalloc(sizeof(kernelConsole_tty),0,0); //Allocate The Console
 
 if(CON == 0x0) { return (kernelConsole_tty*)0; } //Not Allocated!!
 
 //Default Parameters
 if(X <= 0) 
   X = 80;
 if(Y <= 0) 
   Y = 25;
 if(BPP <= 0)
   BPP = 2;
 
 CON->CX   = 0x0;
 CON->CY   = 0x0;
 CON->X    = X;
 CON->Y    = Y;
 CON->BPP  = BPP;
 CON->SIZE = (X*Y*BPP);
 CON->PTR  = (WORD*)(mmMalloc(CON->SIZE,0,0));
 CON->TYPE = 0x1;
 CON->SCROLL = SCROLL;
 CON->COLOR = COLOR;
 
 //Buffer not Allocated
 if(CON->PTR == 0x0) { mmFree((void*)CON); return(kernelConsole_tty*)0; }
 
 Memsetb(CON->PTR,COLOR,CON->SIZE);
 
 return CON;
}

void kernelConsole_updatecursor(kernelConsole_tty*  Ptr)
{
 DWORD Temp = (Ptr->CY*Ptr->X)+Ptr->CX;
 
 kernelProcessorOutPort8(0x3D4,14);
 kernelProcessorOutPort8(0x3D5,(Temp >> 8));
 
 kernelProcessorOutPort8(0x3D4,15);
 kernelProcessorOutPort8(0x3D5,(Temp & 0xFF));
}

void kernelConsole_kill(kernelConsole_tty* Ptr)
{
 if(Ptr == 0x0) { return; }
 
 mmFree((void*)Ptr->PTR);
 mmFree((void*)Ptr);
}

void kernelConsole_draw(kernelConsole_tty* Ptr)
{
 if(Ptr == 0x0) { return; }
 
 Memcpy((void*)0xB8000,(void*)Ptr->PTR,Ptr->SIZE);
 
 Current = Ptr;  //Set To Current
}

void kernelConsole_back(kernelConsole_tty* Ptr)
{
 if(Ptr == 0x0) { return; }

 Memcpy((void*)Ptr->PTR,(void*)0xB8000,Ptr->SIZE);

}

void kernelConsole_clear(kernelConsole_tty* Ptr)
{
 if(Ptr == 0x0) { return; }

 Ptr->CX = 0x0;
 Ptr->CY = 0x0;

 Memsetw((void*)Ptr->PTR,Ptr->COLOR,Ptr->SIZE/0x2);
}

void kernelConsole_cursor(kernelConsole_tty* Ptr,BYTE Mode)
{
 BYTE Cursor;
 
 if(Mode == 0x0) {
    kernelProcessorOutPort8(0x3D4,10);

	kernelProcessorInPort8(0x3D5,Cursor);
	Cursor = Cursor | 0x20;
	kernelProcessorOutPort8(0x3D5,Cursor);
 }else if(Mode == 0x1) { 
    kernelProcessorOutPort8(0x3D4,10);
 
    kernelProcessorInPort8(0x3D5,Cursor);
    Cursor = Cursor & ~0x20;

    kernelProcessorOutPort8(0x3D5,Cursor);
 }
 
 kernelConsole_updatecursor(Ptr);
}

void kernelConsole_putchar(kernelConsole_tty* Ptr,BYTE Chr)
{
 switch(Chr) {
	 case '\b':
		   if(Ptr->CX > 0x0) { Ptr->CX--; }
		   break;
	 case '\t':
		   Ptr->CX = (Ptr->CX+0x4) & ~(4-1);
		   break;
	 case '\r':
		   Ptr->CX = 0x0;
		   break;
	 case '\n':
		   Ptr->CX = 0x0;
		   Ptr->CY++;
		   break;
	 default:
		   if(Chr < ' ') break;
	

		   *(Ptr->PTR+(Ptr->CY*Ptr->X+Ptr->CX)) = (Ptr->COLOR | Chr);
		   //Printf("PTR: %u=%u : %u\n",Ptr->PTR+(Ptr->CY*Ptr->X+Ptr->CX),*(Ptr->PTR+(Ptr->CY*Ptr->X+Ptr->CX)),Ptr->COLOR | Chr);

		   Ptr->CX++;
 }
 
 if(Ptr->CX >= Ptr->X) { Ptr->CX = 0x0; Ptr->CY++; }
 
 kernelConsole_updatecursor(Ptr);
 
 if(Ptr->SCROLL >= 0x1) 
    kernelConsole_scroll(Ptr);
 
}


void kernelConsole_scroll(kernelConsole_tty* Ptr)
{
 if(Ptr->CY >= Ptr->Y) {
    int Offset = Ptr->X;
	WORD *Memory = (WORD*)Ptr->PTR;
	
	while(Offset < (Ptr->X*Ptr->Y*Ptr->BPP)) { 
	  Memory[Offset-Ptr->X] = Memory[Offset];
	  Offset++; 
	}
	
	Offset = (Ptr->X*Ptr->Y*Ptr->BPP) - (Ptr->X*Ptr->BPP);
	
	while(Offset < (Ptr->X*Ptr->Y*Ptr->BPP)) { 
	  Memory[Offset] = 0x20 | Ptr->COLOR;
	  Offset += 2;
    }

	Ptr->CX = 0x0;
	Ptr->CY = Ptr->Y-1;
 }
}

void kernelConsole_putstr(kernelConsole_tty* Ptr,STRING Text)
{
 if(Ptr == 0x0) { return; }
 
 while(*Text)  
   kernelConsole_putchar(Ptr,*Text++);
}

kernelConsole_tty* kernelConsole_current()
{
 return Current;
}

void kernelConsole_color(kernelConsole_tty* Ptr,WORD Color)
{
 if(Ptr == 0x0) { return; }
 
 Ptr->COLOR = Color;
}

unsigned kernelConsole_waitkey(kernelConsole_tty* VC,char* Buffer,BYTE Key,BYTE Print)
{
 unsigned char rET = 0;
 unsigned Size = 0;
 
 while((rET != Key))
 {
  rET = Keyboard_GetKey();  

  if((rET >= 32) && (rET <= 122))
   {
    Size++;
	
	if(Print == 0x1) {
	  kernelConsole_putchar(VC,rET);
	  kernelConsole_draw(VC);
	}
	
	Strncat(Buffer,(char*)&rET,1);
   }
   
 }

 if(Size == 0) return 0;
 else return Size;
 
}
